import re
import json
import os
from pathlib import Path

class Metadata(object):

    __paths = {'QPS': 'RqxMetadata.json',
               'FGC': 'FgcMetadata.json',
               'RB': 'RbMetadata.json',
               'RQ': 'RqMetadata.json',
               'RQX': 'RqxMetadata.json',
               'IPQD2': 'Ipqd2Metadata.json',
               'IPQD4': 'Ipqd4Metadata.json',
               'IPQD8': 'Ipqd8Metadata.json',
               }

    @staticmethod
    def createMetadata(metadataKey):

        selectedPath = os.path.dirname(__file__)
        try:
            selectedPath = os.path.join(selectedPath, Metadata.__paths[metadataKey.upper()])
        except KeyError:
            raise KeyError("{} is not mapped to a json file".format(metadataKey))
              
        return Metadata.__parseJsonFile(selectedPath)

    @staticmethod
    def parseMetadata(metadata, *keys):

        if len(keys) < 1:
            raise ValueError("keys can not be empty!")

        if keys[0] == '*':
            return metadata

        for key in keys:
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist".format(key))

        if len(keys) == 1:
            key = keys[0]
            return {key: metadata[key]}

        return {key: metadata[key] for key in keys}

    @staticmethod
    def getArcNamesFromMagnetNames(identifier, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)['magnetNameToArc']
            return Metadata.__getArcNamesFromMagnetNames(metadata, *keys)
        except KeyError as e:
            raise KeyError(str(e) + "in object parsed from {}".format(Metadata.__paths[identifier]))

    
    @staticmethod
    def getMetadata(identifier, *keys):
        if len(keys) == 1 and isinstance(keys[0], list):
            keys = keys[0]

        try:
            metadata = Metadata.createMetadata(identifier)
            return Metadata.parseMetadata(metadata, *keys)
        except KeyError as e:
            raise KeyError(str(e) + ' in object parsed from {}'.format(Metadata.__paths[identifier]))

    @staticmethod
    def __getArcNameFromMagnetName(metadata, key):
        regExp = "[LR]{1}\d{1}"
        foundRegExp = re.findall(regExp, key)
        if len(foundRegExp) != 1:
            raise ValueError("Key {} does not contain proper MB magnet name".format(key))
        else:
            key = foundRegExp[0]
            if key not in metadata.keys():
                raise KeyError("Key {} does not exist".format(key))
            else:
                return metadata[key]

    @staticmethod
    def __getArcNamesFromMagnetNames(metadata, *keys):

        if len(keys) == 1:
            return Metadata.__getArcNameFromMagnetName(metadata, keys[0])
        else:
            return [Metadata.__getArcNameFromMagnetName(metadata, key) for key in keys]

    @staticmethod
    def __parseJsonFile(fileName):
        
        try:
            with open(fileName) as json_file:
                return json.load(json_file)
        except:
            print("could not load {}".format(fileName))

